% CREATEPLANT.M ---> part of TREE (Math for Biologists Software)
% 
% This file creates plants (in a struct) and initializes some of the 
% values.
% 
% EACH PLANT WILL KNOW:
%     1) Its current string
%     2) Its Rule
%     3) How many generations it has gone through (after first two stops
%     growth)
%     4) Its energy matrix
%     5) Its physical position
%
% Chris Quinn cjq3@cornell.edu last modified


function plant = createplant(curstring, rule1, rule2, pos, IDNUM)
%if is made from two seeds, need rule from seeds and position

plant.id = IDNUM;

%If am not given a string (DIFF from Rule) to start off with, use basic one
if(curstring ~= [])
    plant.curstring = curstring;
else
    plant.curstring = 'F{10}';
end

%If not given a rule ;; note, when click startnew button, should pass in
%the "start rule" as rule 1
if (numel(rule1) ==0 & numel(rule2) == 0)
    fprintf('\nboth rules are empty');

elseif ((numel(rule1) >0) & (numel(rule2) > 0))
    plant.rule = Mixrule(rule1,rule2);

else %assuming only rule is in 1;; as a result of being first generation
    plant.rule = rule1;
end

plant.whichgen = 1;  %I think this might be an artifact from earlier version

if (numel(pos) == 0)
    plant.pos = [0 0]; %%will know if initialization fails if doesn't work
else
    plant.pos = pos;
end

plant.isdead = 'ALIVE';

%energy
plant.energy.cur = 1500; %had going into first round
plant.energy.age = 0; %after one round is one year old
plant.energy.numexppixels = 0; %during round calculate this
plant.energy.numseeds = 0;

%Color is modulated so that user can easily differentiate between plants

%FOR ERROR CHECKING
%plant.color = [IDNUM (255 - IDNUM) IDNUM];

plant.color = [mod(30*IDNUM,256) mod((255 - 30*IDNUM),256) mod(30*IDNUM,256)];

%%maximum of 256^3 = 16,777,216 simultaneous plants


function rule = Mixrule(rule1, rule2)
%%%%%%%
% NOTE:  THIS IS ONE OF THE HARDEST THINGS TO DEFINE!!
%
% Because this is so different than biological organisms, this part of the
% program is difficult to decide how to implement.  So here are my rules:
%
% 1) angles do not really matter (theta angles just rotate; phi angles
% tilt, but not a big deal.
% 2) parsing growth length formulas is hard (at this point don't want to
% have to parse '1.2*(.8*(n+5))' and mix with another one
% 3)branches need to either be kept or discarded (although "merging" the
% branch structure might be ideal, for simplicity, again, keep it as it is
%
% SO: with this pretty much step along, deciding whether to take from
% parent one or two, keeping branches and growths and '/+' as single units
%
% IN THIS VERSION, KEEP IT NICE AND SIMPLE!!! --> the ordering is kept
% constant, with a 50/50 decision between which parent to take the next
% item from, and then discard those two.
%
% 
% 

rule = '';
pointer_1 = 1;%whereIam in rule1
pointer_2 = 1;%whereIam in rule2

while(pointer_1 <= numel(rule1) & pointer_2 <= numel(rule2))

    %FIRST find what the next item will be

    str_1 = ''; %the next item in rule1
    str_2 = ''; %the next item in rule2

    if(rule1(pointer_1) == 'F')
        %find pos of closing brace

        closers = find(rule1 == '}');
        nextclosers = closers(find(closers > pointer_1));
        nextcloser = nextclosers(1);

        str_1 = rule1(pointer_1:nextcloser); %'F{...}'

        pointer_1 = nextcloser + 1;


    elseif(rule1(pointer_1) == '[')
        %find closing bracket

        closers = find(rule1 == ']');
        nextclosers = closers(find(closers > pointer_1));
        nextcloser = nextclosers(1);

        str_1 = rule1(pointer_1:nextcloser); %'F{...}'

        pointer_1 = nextcloser + 1;


    elseif(rule1(pointer_1) == '/')

        str_1 = rule1(pointer_1: pointer_1 + 1);

        pointer_1 = pointer_1 + 2;


    else
        %SHOULD only be an theta change

        str_1 = rule1(pointer_1);
        pointer_1 = pointer_1 + 1;
    end



    %second string
    if(rule2(pointer_2) == 'F')
        %find pos of closing brace

        closers = find(rule2 == '}');
        nextclosers = closers(find(closers > pointer_2));
        nextcloser = nextclosers(1);

        str_2 = rule2(pointer_2:nextcloser); %'F{...}'

        pointer_2 = nextcloser + 1;


    elseif(rule2(pointer_2) == '[')
        %find closing bracket

        closers = find(rule2 == ']');
        nextclosers = closers(find(closers > pointer_2));
        nextcloser = nextclosers(1);

        str_2 = rule2(pointer_2:nextcloser); %'F{...}'

        pointer_2 = nextcloser + 1;


    elseif(rule2(pointer_2) == '/')

        str_2 = rule2(pointer_2: pointer_2 + 1);

        pointer_2 = pointer_2 + 2;


    else
        %SHOULD only be an theta change

        str_2 = rule2(pointer_2);
        pointer_2 = pointer_2 + 1;
    end




    %SECOND decide which should be added and do that

    if(rand <= .5)
        %go with first

        rule = [rule str_1];
    else
        rule = [rule str_2];
    end

end


%now if there was anything left over (would only be on a single parent),
%then 50/50 add it on.


if (pointer_1 <= numel(rule1))



    while(pointer_1 <= numel(rule1))

        %FIRST find what the next item will be

        str_1 = ''; %the next item in rule1


        if(rule1(pointer_1) == 'F')
            %find pos of closing brace

            closers = find(rule1 == '}');
            nextclosers = closers(find(closers > pointer_1));
            nextcloser = nextclosers(1);

            str_1 = rule1(pointer_1:nextcloser); %'F{...}'

            pointer_1 = nextcloser + 1;


        elseif(rule1(pointer_1) == '[')
            %find closing bracket

            closers = find(rule1 == ']');
            nextclosers = closers(find(closers > pointer_1));
            nextcloser = nextclosers(1);

            str_1 = rule1(pointer_1:nextcloser); %'F{...}'

            pointer_1 = nextcloser + 1;


        elseif(rule1(pointer_1) == '/')

            str_1 = rule1(pointer_1: pointer_1 + 1);

            pointer_1 = pointer_1 + 2;


        else
            %SHOULD only be an theta change

            str_1 = rule1(pointer_1);
            pointer_1 = pointer_1 + 1;
        end





        %SECOND decide which should be added and do that

        if(rand <= .5)
            %added

            rule = [rule str_1];
        end

    end




elseif(pointer_2 <= numel(rule2))


    while(pointer_2 <= numel(rule2))

        %FIRST find what the next item will be

        str_2 = ''; %the next item in rule2


        %second string
        if(rule2(pointer_2) == 'F')
            %find pos of closing brace

            closers = find(rule2 == '}');
            nextclosers = closers(find(closers > pointer_2));
            nextcloser = nextclosers(1);

            str_2 = rule2(pointer_2:nextcloser); %'F{...}'

            pointer_2 = nextcloser + 1;


        elseif(rule2(pointer_2) == '[')
            %find closing bracket

            closers = find(rule2 == ']');
            nextclosers = closers(find(closers > pointer_2));
            nextcloser = nextclosers(1);

            str_2 = rule2(pointer_2:nextcloser); %'F{...}'

            pointer_2 = nextcloser + 1;


        elseif(rule2(pointer_2) == '/')

            str_2 = rule2(pointer_2: pointer_2 + 1);

            pointer_2 = pointer_2 + 2;


        else
            %SHOULD only be an theta change

            str_2 = rule2(pointer_2);
            pointer_2 = pointer_2 + 1;
        end

        %SECOND decide which should be added and do that

        if(rand <= .5)
            %add it
            rule = [rule str_2];
        end

    end


end

