% GROW.M ---> part of TREE (Math for Biologists Software)
% 
% This file takes care of taking the current string and applying the
% plant's rule to it.  Currently, each plant only has a single rule(for
% simplicity) and so if need be, could do a stochastic rule (any of several
% rules chosen to be applied to a corresponding string.
% 
% Chris Quinn cjq3@cornell.edu last modified


function plant = grow(plant,handles)
%This file will take a plant, and if it has enough energy, will have it
%grow, else will just return

%USER DEFINED VARIABLES
prob_will_grow = .95; %scale 0-1  probability any given F will be exchanged for the rule
penalty_size = 2;  %so will find its total length, multiply by two, and subtract that
energy_to_grow = 450;
max_age_to_grow = 2;


%If want it to printout 
canprint = get(handles.Radio_Printout_Calcs,'Value');


%%%%%%%%%%% CHECK IF PLANT IS ALLOWED TO GROW %%%%%%%%%%%%%%%%%%%%%

%Rule : plant must have at least 250 energy to grow, and be under max age)

if (plant.energy.cur < energy_to_grow | plant.energy.age > max_age_to_grow)
    if(canprint)
        fprintf('\nPlant #%d cannot grow', plant.id);
    end
    return
end




%%%%%%%%%%%%%%%%%% IF CAN GROW %%%%%%%%%%%%%%%%%%%%

%Rule : growing costs 450 energy

plant.energy.cur = plant.energy.cur -energy_to_grow;



%$$$$$$$$$$$$$$$$    GROW IT   $$$$$$$$$$$$$$$$%

tempstring = plant.curstring;
temprule = plant.rule;


%Rule : Whether or not a rule is applied is a probability
%Rule : The whole rule is applied for a given F (so if it is applied, the
%       whole rule replaces that F


%Find all the F's and left brackets (latter to get branch sizes)  'F{343}'
where_F = find(tempstring == 'F');
where_rightcurl = find(tempstring == '}');

if(canprint)
    fprintf('\nPlant #%d\tBefore:\n%s', plant.id,tempstring);
end

for ii = 1:numel(where_F)

    %randomly decide if this F will have rule applied
    if (rand < prob_will_grow)

        %first find the length of the current branch by finding {}

        next_right_curls = where_rightcurl(find(where_rightcurl > where_F(ii)));
        next_right_curl = next_right_curls(1); %so this is the next one;

        %then get the string representing its length
        length = tempstring((where_F(ii) + 2):(next_right_curl - 1));

        %now take the rule, replace the 'n' with length
        new_temprule = regexprep(temprule, 'n', length);


        temp_length = numel(tempstring);
        %now plug into tempstring

        if(where_F(ii) ~=1)

            tempstring = [tempstring(1:(where_F(ii) - 1)) new_temprule tempstring(next_right_curl+1:end)];
        else
            tempstring = [new_temprule tempstring(next_right_curl+1:end)];
        end

        %now update the positions of the original 'F'

        where_F = where_F + (numel(tempstring) - temp_length);
        where_rightcurl = where_rightcurl + (numel(tempstring) - temp_length);

    else
        %fprintf('\nPlant #%i did not grow by chance', plant.id);
    end

end

if(canprint)
    fprintf('\nWith Rule: %s', plant.rule);
    fprintf('\nPlant #%d\tAfter:\n%s\n', plant.id,tempstring);
end


plant.curstring = tempstring;


%% PENALTY FOR SIZE WHEN GROW%%

numfs = find(strcmp(plant.curstring, 'F'));
size = 0;

for ii = 1:numfs
    
    %find next '}'
    closer = find(strcmp(plant.curstring, '}'));
    nextclosers = closer(find(closer > ii));
    nextcloser = nextclosers(1);   
    
    
    
    %get length add it to total
    
    tempsize = str2num(plant.curstring(closer + 1: nextcloser-1));
    
    size = size + tempsize;
    
end

plant.energy.cur = plant.energy.cur - size^(penalty_size);













