% RUNSTEP.M ---> part of TREE (Math for Biologists Software)
% 
% Process
%   1) Grow all of the plants
%   2) Draw all of the plants in user cam and light cam
%   3) Calculate number of exposed pixels
%   4) Calculate energies
%   5) Update plant array with the new plants/remove dead ones
%
% Chris Quinn cjq3@cornell.edu last modified

function [plantarray theaxes newid] = runstep(plantarray, theaxes, handles, newid)


%%%%%%%%%%%%%%%%  VARS
%
% newid = the id num that will be used for the next plant (increments from
% 1;   once hit 255 notify user since the colors for plants will no longer
% be necessarily unique --> change this once get other things settled.
%
%

%see if ok to printout things

canprint = get(handles.Radio_Printout_Calcs,'Value');

NUMPLANTS = numel(plantarray);
energyperseed = 400;  %the amount of energy units a plant needs to produce a single seed; note this is NOT subtracted from energy
seedingdistance = 3; %the x and y sizes for the square plot a seed can fall within
chanceseedwillgrow = 50; % 0 - 100 (%) chance any two seeds with potential to grow actually will
max_seed_separation = 2; %number of length units two seeds can be from each other and still be able to grow
age_penalty = 150; %energy = energy - age_penalty*age;
penalty_size = 1.5; % so subtract totallength*penalty_size

if(NUMPLANTS == 0)
    errordlg('The Plants have all died!!');
    return;
end

if(canprint)
    fprintf('\n\n************ BEGINNING A ROUND *************\n');
end


%%%%%%%%%%%%%%%%   REDRAW THE PLANTS %%%%%%%%%%%%%%%%%%%%

%First get camera positions so can return to same view
user_axes = theaxes.user;
light_axes = theaxes.light;


%first make sure there is a valid value in num steps
%so far, these are only two manually controlled parameters
user_cam.CameraPosition = get(user_axes, 'CameraPosition');
user_cam.CameraUpVector = get(user_axes, 'CameraUpVector');

light_cam.CameraPosition = get(light_axes, 'CameraPosition');
light_cam.CameraUpVector = get(light_axes, 'CameraUpVector');


%redraw the axes
delete(user_axes);
user_axes = axes('parent', handles.Display); %this will clear any previous ones

delete(light_axes);
light_axes = axes('parent', handles.light_source);

hold(light_axes,'on');
hold(user_axes,'on');

if(canprint)
    fprintf('########   GROWING THE PLANTS ######')
end


%First, pass through all the plants through string generator, increase the
%age of each, and then draw for user

for ii = 1:NUMPLANTS
    plantarray{ii}.energy.cur = plantarray{ii}.energy.cur - age_penalty*plantarray{ii}.energy.age;%10^plantarray{ii}.energy.age; %it costs to grow but right now only depends on age, not complexity
    %note, there is a proportional penalty for age AND a constant penalty for growing

    if (plantarray{ii}.energy.cur <=0)

        if(canprint)
            fprintf('\nThis plant is dead:\tID:%d\t%s',plantarray{ii}.id, plantarray{ii}.curstring);
        end

        plantarray{ii}.isdead = 'DEAD'; %CHECK THIS or should I draw in black?

    else %else regrow and draw

        if (plantarray{ii}.energy.age <2) %only grow for the first two years
            plantarray{ii} = grow(plantarray{ii},handles);
        else
            %if don't grow, at least penalize some for the size
            %% PENALTY FOR SIZE %%
            
            numfs = find(strcmp(plantarray{ii}.curstring, 'F'));
            size = 0;

            for gg = 1:numfs

                %find next '}'
                closer = find(strcmp(plantarray{gg}.curstring, '}'));
                nextclosers = closer(find(closer > gg));
                nextcloser = nextclosers(1);

               %get length add it to total

                tempsize = str2num(plantarray{gg}.curstring(closer + 1: nextcloser-1));

                size = size + tempsize;

            end

            plantarray{ii}.energy.cur = plantarray{ii}.energy.cur - penalty_size*size;
            
        end

        plantarray{ii}.energy.age = plantarray{ii}.energy.age + 1;

        axes(light_axes); %set current axes as the light_axes
        hold on
        drawplant(plantarray{ii}, light_axes);

        axes(user_axes);
        hold on
        drawplant(plantarray{ii}, user_axes);
    end

end

%return to the user's camera angle after drawing

set(user_axes, 'CameraPosition', user_cam.CameraPosition);
set(user_axes, 'CameraUpVector', user_cam.CameraUpVector);

set(light_axes, 'CameraPosition', light_cam.CameraPosition);
set(light_axes, 'CameraUpVector', light_cam.CameraUpVector);


%To ensure that all parts of plants are within the axes border and so are
%counted in the pixel count.
X_Lim = get(light_axes,'XLim');
set(light_axes,'XLim',[0 100]);
X_ticks = get(light_axes,'XTick');
set(light_axes,'XTick',0:10:100);

Y_Lim = get(light_axes,'YLim');
set(light_axes,'YLim',[0 100]);
Y_ticks = get(light_axes,'YTick');
set(light_axes,'YTick',0:10:100);

% Z_Lim = get(light_axes,'ZLim');
% set(light_axes,'ZLim',[0 (Z_Lim(end) + 2)]);
Z_ticks = get(light_axes,'ZTick');
set(light_axes,'ZTick',0:5:(Z_ticks(end)));

set(light_axes,'XGrid', 'on');
set(light_axes,'YGrid', 'on');


axes(user_axes);
rotate3d(user_axes, 'ON');

theaxes.light = light_axes;
theaxes.user = user_axes;

if(canprint)
    fprintf('\n$$ I have drawn the plants and am counting pixels and energies $$\n');
end

%after having drawn the plants, now calculate numexp pixels for each color
%--> since color not based on genetics, if two plants share same color,
%should not be too much of a problem -- if anything adds a little
%randomness to the system XXX once get other things settled, make sure to
%allow for 1 -255 first in green, then 1-255 additionally in red, ... so
%first plant [0 0 0], 230th plant [0 230 0], ...????

position = get(handles.light_source,'Position');

%add a 10 unit border because I think that there are problems with getting
%pixel count see 3.13.06_4.30_problem_with_pixcount.jpeg
left = position(1)-10;
bottom = position(2)-10;
width = position(3)+10;
height = position(4)+10;

userframe = getframe(light_axes);
pix = userframe.cdata;
%pix = allpix( find( (allpix(:,1,:)> bottom + height) & (allpix(:,1,:) < bottom) & (allpix(:,2,:) > left + width) & (allpix(:,2,:) < left)));  %this might be off by one pixel in vert and horiz directions

%isdead = numel(plantarray == 'DEAD');
curseed = 1; %place holder  if plant 4 has 5seeds, 5 has 3 seeds, then in numseeds [... 4 4 4 4 4 5 5 5 ...]

seeding = zeros(NUMPLANTS * 20, 3);%but shouldn't have more than 5 seeds /plant at any one time

%PROBLEM : Matlab's cdata means as long as is within whole figure, counts.

%need an index for which living plant this is (below, when access "seed_1",
%that refers to the plant.id, not which of the indexed plants it is, which
%is how it accesses, so, to avoid this problem, keep a tally of which
%living plant this is with reference to order in plantarray
whichlivingplant = 1;

for ii = 1:NUMPLANTS

    curcolor = plantarray{ii}.color;
    plantarray{ii}.energy.numexppixels = numel(find(pix(:,:,1) == curcolor(1) & pix(:,:,2) == curcolor(2) & pix(:,:,3) == curcolor(3)));

    %Now calculate which have died, which have produced seeds, mix the seeds
    %available and put into spot of plants that have died, printing out reports
    %along the way

    %These coefficients are arbitrary --> should test until like
    %  Already have an age penalty   plantarray{ii}.energy.cur = plantarray{ii}.energy.cur - 5^(plantarray{ii}.energy.age);

    energperpix = get(handles.Energy_slider, 'Value');
    
    plantarray{ii}.energy.cur = plantarray{ii}.energy.cur + round(energperpix*plantarray{ii}.energy.numexppixels);


    %%NOW UPDATE "SEEDING" --> every seed is a new row, data in respective
    %%columns
    tempseeds = floor(plantarray{ii}.energy.cur / energyperseed);
    if (tempseeds < 0)
        tempseeds = 0;
    end



    plantarray{ii}.energy.numseeds = tempseeds;
    for tt =1:tempseeds

        %for simplicity (until have time to develop more fully), let each
        %seed fall within a certain radius

        xpos = round(rand*(seedingdistance) + (plantarray{ii}.pos(1) - 1/2*(seedingdistance)));
        ypos = round(rand*(seedingdistance) + (plantarray{ii}.pos(2) - 1/2*(seedingdistance)));


        seeding(curseed, 1:3) = [whichlivingplant xpos ypos];

        curseed = curseed + 1;

        clear xpos ypos
    end

    %
    %     numseeds(curseed:(curseed + plantarray{ii}.energy.numseeds - 1)) = ii;
    %     curseed = curseed + plantarray{ii}.energy.numseeds;
    %



    if (plantarray{ii}.energy.cur < 0)
        died = 'yes';
        isdead(ii) = 1;
    else
        died = 'no';
        isdead(ii) = 0;
        whichlivingplant = whichlivingplant + 1;
    end

    if(canprint)
        fprintf('\nPLANT # %i\tAGE = %i\tENERGY = %i\tDIED = %s\tPIX = %5.0i\tNUMSEEDS = %i', plantarray{ii}.id,plantarray{ii}.energy.age, plantarray{ii}.energy.cur, died, plantarray{ii}.energy.numexppixels,plantarray{ii}.energy.numseeds);
    end

end
%
% results.isdead = isdead;
% results.numseeds = numseeds(find(numseeds));
% results.theplants = plantarray;

if(canprint)
    fprintf('\n$$ Im done with calcs --> now will delete the dead plants and create the new ones $$\n\n');
end


%DELETE THE DEAD plants

notdone = 1;
curplant = 1;

while(curplant<=numel(plantarray))

    %if current one is dead
    if (strcmp(plantarray{curplant}.isdead, 'DEAD'))

        %shift the seeding array (for all of the plants that were after
        %this, they are now moved up
        
        whichseeds = find(seeding(:,1) >= curplant); %although they should never be equal
        
        seeding(whichseeds,1) = seeding(whichseeds,1) - 1;  %NOT SURE IF THIS DOES WHAT I THINK IT DOES     
        
        plantarray(curplant) = [];
        
           

        %if current one is alive
    else
        curplant = curplant+1;
    end


end

clear notdone curplant




%%% MAKING NEW PLANTS
%
% Since at this point I want to get a working version out, make a simple
% seeding and growth model.  Seeds can land anywhere within a square plot
% with the plant at center.  Then want to randomly pick a seed, see if any
% other seeds are close, if so pick another seed from the latter group,
% randomly decide if they will grow a new plant; whether or not delete from
% the "seeding"
%
%


%first delete all of the zero entries

firstzeros = find(seeding(:,1) == 0);
firstzero = firstzeros(1);

if (firstzero > 1)
    seeding = seeding(1:(firstzero-1),:);
else
    fprintf('\nNO SEEDS\n');
    seeding = [];
end



newplants = 0;

if(numel(seeding) > 0)
    numseeds = numel(seeding(:,1));

    while(numseeds >=2)

        %randomly pick a seed and extract it from the list
        whichseed = ceil(rand*numseeds);
        seed_1 = seeding(whichseed,:);

        if (whichseed == 1)
            seeding = [seeding(2:end,:)];
        else
            seeding = [seeding(1:whichseed-1,:); seeding((whichseed + 1):end, :)];
        end



        %now find all the others that are close to it and pick one of them,
        %extracting it from the list


        potential_seeds = find( [(seeding(:,2) - seed_1(2))].^2 + [(seeding(:,3) - seed_1(3))].^2 <=max_seed_separation);

        whichseed = ceil(rand*numel(potential_seeds));

        if(whichseed ~=0) %only if there are nearby seeds

            seed_2 = seeding(whichseed, :);

            if (whichseed == 1)
                seeding = [seeding(2:end,:)];
            else
                seeding = [seeding(1:whichseed-1,:); seeding((whichseed + 1):end, :)];
            end


            %now decide if this will make a plant or not

            if(rand <= chanceseedwillgrow/100)

                %for debugging since had errors with below lines
                
                if(numel(seed_1) < 1 | numel(seed_2) < 1)
                    disp('there is something wrong with seeding');
                elseif(numel(plantarray) < seed_1(1) | numel(plantarray) < seed_2(1))
                        disp('there is something else wrong with seeding');
                end
                
                
                try
                rule1 = plantarray{seed_1(1)}.rule; %this line has caused problems becauseseed_1(1) used to be plant.id not its place in array; that should have been fixed with 'whichlivingplant'
                rule2 = plantarray{seed_2(1)}.rule;
                catch
                    disp('ERROR WITH SEEDING')
                end
                xpos = round((seed_1(2) + seed_2(2))/2);
                ypos = round((seed_1(3) + seed_2(3))/2);

                tempnumplants = numel(plantarray);

                plantarray{tempnumplants + 1} = [createplant([],[rule1],[rule2],[xpos ypos],newid)];

                %temnewplants = newplants +1;

                newid = newid + 1;

                clear rule1 rule2 xpos ypos tempnumplants

            end

        end

        numseeds = numel(seeding(:,1));
    end
end

if(canprint)
    fprintf('\nThere are %i new plants\n', newplants);

    fprintf('\nTHERE ARE NOW ::  %d  PLANTS IN THE FIELD\n', numel(plantarray));

    fprintf('\nNow will morph\n');
end

percent_morph = str2num(get(handles.Edit_percent_morph, 'String'));

plantarray = morph(plantarray, percent_morph);

if(canprint)
    fprintf('\n************ END OF ROUND *************\n\n')
end

%%%REDRAW SO USER CAN SEE THE NEW PLANTS
%%EACH PLANT STARTS OUT THE SAME: 'F{10}'


%first make sure there is a valid value in num steps
%so far, these are only two manually controlled parameters
user_cam.CameraPosition = get(user_axes, 'CameraPosition');
user_cam.CameraUpVector = get(user_axes, 'CameraUpVector');

light_cam.CameraPosition = get(light_axes, 'CameraPosition');
light_cam.CameraUpVector = get(light_axes, 'CameraUpVector');


%redraw the axes
delete(user_axes);
user_axes = axes('parent', handles.Display); %this will clear any previous ones

delete(light_axes);
light_axes = axes('parent', handles.light_source);

hold(light_axes,'on');
hold(user_axes,'on');
%
% fprintf('########   GROWING THE PLANTS ######')

NUMPLANTS = numel(plantarray);

for ii = 1:NUMPLANTS
    % plantarray{ii}.energy.cur = plantarray{ii}.energy.cur - 350;%10^plantarray{ii}.energy.age; %it costs to grow but right now only depends on age, not complexity
    %note, there is a proportional penalty for age AND a constant penalty for growing


    %if (plantarray{ii}.energy.cur <=0)
    %         plantarray{ii}.curstring = Morph(plantarray{ii});
    % ('This plant is dead:')
    % disp(plantarray{ii}.curstring);

    % plantarray{ii}.isdead = 'DEAD'; %CHECK THIS or should I draw in black?

    % else %else regrow and draw

    % if (plantarray{ii}.energy.age <2) %only grow for the first two years
    %     plantarray{ii} = grow(plantarray{ii});
    % end


    %plantarray{ii}.energy.age = plantarray{ii}.energy.age + 1;

    axes(light_axes); %set current axes as the light_axes
    hold on
    drawplant(plantarray{ii}, light_axes);

    axes(user_axes);
    hold on
    drawplant(plantarray{ii}, user_axes);
    % end

end

%return to the user's camera angle after drawing

set(user_axes, 'CameraPosition', user_cam.CameraPosition);
set(user_axes, 'CameraUpVector', user_cam.CameraUpVector);

set(light_axes, 'CameraPosition', light_cam.CameraPosition);
set(light_axes, 'CameraUpVector', light_cam.CameraUpVector);



%To ensure that all parts of plants are within the axes border and so are
%counted in the pixel count.
X_Lim = get(light_axes,'XLim');
set(light_axes,'XLim',[0 100]);
X_ticks = get(light_axes,'XTick');
set(light_axes,'XTick',0:10:100);

Y_Lim = get(light_axes,'YLim');
set(light_axes,'YLim',[0 100]);
Y_ticks = get(light_axes,'YTick');
set(light_axes,'YTick',0:10:100);

% Z_Lim = get(light_axes,'ZLim');
% set(light_axes,'ZLim',[0 (Z_Lim(end) + 2)]);
Z_ticks = get(light_axes,'ZTick');
set(light_axes,'ZTick',0:5:(Z_ticks(end)));

set(light_axes,'XGrid', 'on');
set(light_axes,'YGrid', 'on');

axes(user_axes)
rotate3d(user_axes, 'ON');

theaxes.light = light_axes;
theaxes.user = user_axes;







